/* eslint-disable */
import {
  Http, IDictionary, Router, Metadata,
} from '@zeedhi/core';
import {
  AlertService, Date as ZdDate, DialogService, Form, LoadingService,  Modal, ModalService, Select,
  SelectMultiple, Tabs, Text, TextInput,
} from '@zeedhi/common';
import { TekGrid } from '@zeedhi/teknisa-components-common';

export class BusinessPartnerController {
  private addNewPartner!: Modal;

  public NAMEOFCORPORATE: string = '';

  private IDPESSOAFISICA: any;

  private IDINSTITUICAO: any;

  public openPreview({ row }: any) {
    Router.replace(`/informacoes_parceiros?${row.NRPARCNEGOCIO}`);
  }

  public clickButtonToExpand({ component }: any) {
    const form = Metadata.getInstance<Form>(component.name.split('_')[1]);
    form.isVisible = !form.isVisible;

    component.iconName = (component.iconName === 'mdi-chevron-down') ? 'mdi-chevron-right' : 'mdi-chevron-down';
  }

  public alertMessage(message: string, type: string) {
    AlertService.show({
      name: 'basic-alert',
      text: message,
      top: true,
      left: false,
      timeout: 3000,
      color: type,
    });
  }

  public dialogMessage(message: string) {
    DialogService.show({
      name: 'warning-dialog',
      type: 'warning',
      title: 'Erro!',
      text: message,
    });
  }

  public validateRegistration(nameSelect: string, nameText: string, valueRegistration: string) {
    const selectTypeRegistration = Metadata.getInstance<Select>(nameSelect);
    let textInputNumberRegistration = Metadata.getInstance<TextInput>(nameText);

    if (valueRegistration) {
      const validations = this.validateRegistrationFormat(valueRegistration);

      textInputNumberRegistration.value = validations.parameter;
      selectTypeRegistration.value = validations.cod;

      if (validations.cod === 'CPF') {
        this.IDPESSOAFISICA = 'S';
        this.IDINSTITUICAO = 'N';
      } else {
        this.IDPESSOAFISICA = 'N';
        this.IDINSTITUICAO = 'S';

        if (validations.cod !== 'CNPJ') textInputNumberRegistration.value = validations.value;
      }

      if (validations.valid === false) {
        selectTypeRegistration.value = null;
        textInputNumberRegistration.value = null;
        this.alertMessage('Número de Inscrição Inválido', 'error');
      }

    } else {
      selectTypeRegistration.value = null;
      textInputNumberRegistration.value = null;
    }
  }

  public validateCPF(CPF: string) {
    if (CPF === null || CPF === undefined) {
      return false;
    } else {
      CPF = CPF.toString().replace(/[\D]+/g, '');

      if (CPF.length != 11) return false;

      if (CPF == "00000000000" ||
  			CPF == "11111111111" ||
  			CPF == "22222222222" ||
  			CPF == "33333333333" ||
  			CPF == "44444444444" ||
  			CPF == "55555555555" ||
  			CPF == "66666666666" ||
  			CPF == "77777777777" ||
  			CPF == "88888888888" ||
  			CPF == "99999999999") return false;

  		let soma = 0;
  		let resto: any;

      for (let i = 1; i <= 9; i++) soma = soma + parseInt(CPF.substring(i - 1, i)) * (11 - i);
      resto = (soma * 10) % 11;

      if ((resto == 10) || (resto == 11)) resto = 0;
      if (resto != parseInt(CPF.substring(9, 10))) return false; //'resto' é o 1º dígito de validação do CPF

      soma = 0;
      for (let i = 1; i <= 10; i++) soma = soma + parseInt(CPF.substring(i - 1, i)) * (12 - i);
      resto = (soma * 10) % 11;

      if ((resto == 10) || (resto == 11)) resto = 0;
      if (resto != parseInt(CPF.substring(10, 11))) return false; //'resto' é o 2º dígito de validação do CPF

      return true;
    }
  }

  public validateCNPJ(CNPJ: string) {
		if (CNPJ === undefined || CNPJ === null) {
		  return false;
		} else {
		  CNPJ = CNPJ.toString().replace(/[\D]+/g, '');

  		if (CNPJ.length != 14) return false;

  		if (CNPJ == "00000000000000" ||
  			CNPJ == "11111111111111" ||
  			CNPJ == "22222222222222" ||
  			CNPJ == "33333333333333" ||
  			CNPJ == "44444444444444" ||
  			CNPJ == "55555555555555" ||
  			CNPJ == "66666666666666" ||
  			CNPJ == "77777777777777" ||
  			CNPJ == "88888888888888" ||
  			CNPJ == "99999999999999") return false;

  		let tamanho: any = CNPJ.length - 2;
  		let numeros: any = CNPJ.substring(0, tamanho);
  		let digitos: any = CNPJ.substring(tamanho);
  		let soma: any = 0;
  		let pos: any = tamanho - 7;

  		for (let i = tamanho; i >= 1; i--) {
  			soma += numeros.charAt(tamanho - i) * pos--;
  			if (pos < 2) pos = 9;
  		}

  		let resultado = (soma % 11 < 2) ? 0 : 11 - soma % 11;
  		if (resultado != digitos.charAt(0)) return false; //'resultado' é o 1º digito de validação do CNPJ

  		tamanho = tamanho + 1;
  		numeros = CNPJ.substring(0, tamanho);
  		soma = 0;
  		pos = tamanho - 7;

  		for (let i = tamanho; i >= 1; i--) {
  			soma += numeros.charAt(tamanho - i) * pos--;
  			if (pos < 2) pos = 9;
  		}

  		resultado = (soma % 11 < 2) ? 0 : 11 - soma % 11;
  		if (resultado != digitos.charAt(1)) return false; //'resultado' é o 2º digito de validação do CNPJ

  		return true;
		}
	}

  public fill(value: string, length: number, toRight: boolean = false, filler: any = '') {
    if (value) {
			value = value.toString();

			if (filler) {
				filler = filler.toString()[0];
			} else filler = '0';

			if (toRight) while (value.length + filler.length <= length) value = value + filler;
			else while (value.length + filler.length <= length) value = filler + value;
		}
		return value;
  }

  public validateRegistrationFormat(registration: string, force: boolean = false) {
    const TypeRegistration: IDictionary = {
      parameter: '',
      value: '',
      valid: null,
      cod: null,
      name: '',
    };

    if (registration === undefined || registration === null || registration === '') {
      return TypeRegistration;
    } else {
      registration = registration.toString().replace(/[\D]+/g, '');

      if (registration.length <= 5) {
        TypeRegistration.cod = 'CODIGO';
        TypeRegistration.name = 'Còdigo genérico';
        TypeRegistration.valid = true;
        TypeRegistration.value = this.fill(registration, 5);
      } else {
        if (registration.length === 11) {
          TypeRegistration.cod = 'CPF';
          TypeRegistration.name = 'Pessoa física';
          TypeRegistration.valid = this.validateCPF(registration);
          TypeRegistration.value = registration.substr(0, 3) + '.' + registration.substr(3, 3) + '.' + registration.substr(6, 3) + '-' + registration.substr(9, 2);
        } else if (registration.length === 14) {
          TypeRegistration.cod = 'CNPJ';
          TypeRegistration.name = 'Pessoa jurídica';
          TypeRegistration.valid = this.validateCNPJ(registration);
          TypeRegistration.value = registration.substr(0, 2) + '.' + registration.substr(2, 3) + '.' + registration.substr(5, 3) + '/' + registration.substr(8, 4) + '-' + registration.substr(12, 2);
        } else force = true;

        if (force && TypeRegistration.valid !== true) {
  				TypeRegistration.cod = 'LIVRE';
  				TypeRegistration.name = 'Inscrição genérica';

  			  if (registration.length <= 20) {
  					TypeRegistration.valid = true;
  					TypeRegistration.value = this.fill(registration, 20);
  			  } else {
  					TypeRegistration.valid = false;
  					TypeRegistration.value = registration.substr(0, 20);
  				}
  			}
      }

      TypeRegistration.parameter = registration;
      return TypeRegistration;
    }
  }

  // ---------------------- Add Business Partner Modal Functions -----------

  public async openModalNewPartner() {
    this.addNewPartner = await ModalService.createFromJson('BusinessPartner/Modal/modal-partner-registration', true);
    this.addNewPartner.show();
  }

  public closeModalNewPartner() {
    this.addNewPartner.hide();
    this.addNewPartner?.destroy();
  }

  /* Função que chama a API dos correios e seta os valores nos campos de endereço
   Faz uma validação buscando o código do bairro no banco de dados pois não é retornado pela API */
  public async validateCEP() {
    const CEP = Metadata.getInstance<TextInput>('BPForm-CEP').value.replace('-', '');

    if (CEP) {
      (this.searchCep(CEP) as any).then(async (response: IDictionary) => {
        const addressFound = response.data.dataset.ENDERECO;

        if (addressFound) {
          Metadata.getInstance<TextInput>('BPForm-address').value = (addressFound.DSENDERECO) ? addressFound.DSENDERECO : '';
          await Metadata.getInstance<Select>('BPForm-country').setValue((addressFound.CDPAIS) ? addressFound.CDPAIS : '');
          await Metadata.getInstance<Select>('BPForm-state').setValue((addressFound.SGESTADO) ? addressFound.SGESTADO : '');
          await Metadata.getInstance<Select>('BPForm-city').setValue((addressFound.CDMUNICIPIO) ? addressFound.CDMUNICIPIO : '');
          if (addressFound.CDBAIRRO === '') {
            Http.post('/findCodeDistrict', {
              NMBAIRRO: addressFound.NMBAIRRO,
              SGESTADO: addressFound.SGESTADO,
              CDPAIS: addressFound.CDPAIS,
              CDMUNICIPIO: addressFound.CDMUNICIPIO,
            }).then((response: IDictionary) => {
              response.data.dataset.codeFound.forEach((codeFound: IDictionary) => {
                Metadata.getInstance<Select>('BPForm-district').setValue((codeFound.CDBAIRRO) ? codeFound.CDBAIRRO : '');
              });
            });
          }
        }
      }).catch((error: any) => {
          if (error.data) this.alertMessage(error.data.error, 'error')
          else this.alertMessage('Erro ao realizar a busca.', 'error');
          Metadata.getInstance<TextInput>('BPForm-CEP').value = '';
          Metadata.getInstance<TextInput>('BPForm-address').value =  '';
          Metadata.getInstance<Select>('BPForm-country').value = '';
          Metadata.getInstance<Select>('BPForm-state').value = '';
          Metadata.getInstance<Select>('BPForm-city').value = '';
          Metadata.getInstance<Select>('BPForm-district').value = '';
      });
    }
    await this.changeStateToEnabled();
  }

  // Faz uma busca pelo CEP através da API do Correio, e retorna o endereço em formato de promise.
  public searchCep(cepValue: number) {
    if (String(cepValue).length === 8) {
      return Http.post('/lib_findAddress', {
        requestType: 'Row',
        row: { CEP: cepValue },
      });
    } else this.alertMessage('CEP Inválido!', 'error')
    return 0;
  }

  /* Função que habilita o campo de Estado caso o campo de País esteja com um valor
   e filtra os estados a partir do País selecionado */
  public async changeStateToEnabled() {
    const selectCountry = Metadata.getInstance<Select>('BPForm-country');
    const selectState = Metadata.getInstance<Select>('BPForm-state');

    if (!selectCountry.value) {
      selectState.disabled = true;
      selectState.value = '';
    } else {
      selectState.disabled = false;
      selectState.datasource.setFilter({
        CDPAIS: selectCountry.value,
      });
    }
    await this.changeCityToEnabled();
  }

  /* Função que habilita o campo de Município caso o campo de Estado esteja com um valor
   e filtra os municípios a partir do Estado selecionado */
  public async changeCityToEnabled() {
    const selectCountry = Metadata.getInstance<Select>('BPForm-country');
    const selectState = Metadata.getInstance<Select>('BPForm-state');
    const selectCity = Metadata.getInstance<Select>('BPForm-city');

    if (!selectState.value) {
      selectCity.disabled = true;
      selectCity.value = '';
    } else {
      selectCity.disabled = false;
      selectCity.datasource.setFilter({
        CDPAIS: selectCountry.value,
        SGESTADO: selectState.value,
      });
    }
    await this.changeDistrictToEnabled();
  }

  /* Função que habilita o campo de Bairro caso o campo de Município esteja com um valor
   e filtra os bairros a partir do Município selecionado */
  public async changeDistrictToEnabled() {
    const selectCountry = Metadata.getInstance<Select>('BPForm-country');
    const selectState = Metadata.getInstance<Select>('BPForm-state');
    const selectCity = Metadata.getInstance<Select>('BPForm-city');
    const selectDistrict = Metadata.getInstance<Select>('BPForm-district');

    if (!selectCity.value) {
      selectDistrict.disabled = true;
      selectDistrict.value = '';
    } else {
      selectDistrict.disabled = false;
      selectDistrict.datasource.setFilter({
        CDPAIS: selectCountry.value,
        SGESTADO: selectState.value,
        CDMUNICIPIO: selectCity.value
      });
    }
  }

  public validateRegistrationOfModalAddNewPartner(){
    const registrationTpe = Metadata.getInstance<Select>('formNewPartner-registrationType').name;
    const registration = Metadata.getInstance<TextInput>('formNewPartner-Nregistration');
    this.validateRegistration(registrationTpe, registration.name, registration.value);
  }

  public insertNewBusinessPartner() {
    const FormBP = Metadata.getInstance<Form>('formNewPartner').validate();
    const FormAddress = Metadata.getInstance<Form>('BPForm').validate();
    const DISTRICT = Metadata.getInstance<Select>('BPForm-district').value;
    const grid = Metadata.getInstance<TekGrid>('gridBusinessPartner');
    const typeRegistration = Metadata.getInstance<Select>('formNewPartner-registrationType').value;

    if (FormBP && FormAddress) {
      Http.post('/insertNewBusinessPartner', {
        CDTIPOINSCRICAO: typeRegistration,
        NRINSCRICAOPARC: Metadata.getInstance<TextInput>('formNewPartner-Nregistration').value,
        IDATIVO: Metadata.getInstance<Select>('formNewPartner-active').value,
        CDTIPOPARCPRINCIPAL: Metadata.getInstance<Select>('formNewPartner-mainType').value,
        NRTPCLASSIFICAPARC: Metadata.getInstance<Select>('formNewPartner-partnerClassification').value,
        CDTIPOPARCEIRO: Metadata.getInstance<Select>('formNewPartner-typeOfPartner').value,
        NMPRINCIPALPARC: Metadata.getInstance<TextInput>('formNewPartner-corporateName').value,
        NMSECUNDARIPARC: Metadata.getInstance<TextInput>('formNewPartner-shortName').value,
        CDOPERRESPONSAVEL: Metadata.getInstance<Select>('formNewPartner-operator').value,
        DTNASCIFUNDPARC: Metadata.getInstance<ZdDate>('formAdditionalData-initialDate').value,
        CDSEGMER: Metadata.getInstance<Select>('formAdditionalData-markedSegment').value,
        CDATIVCLI: Metadata.getInstance<Select>('formAdditionalData-branchOfActivity').value,
        NRSEQTPATIVCLI: Metadata.getInstance<Select>('formAdditionalData-typeOfActivity').value,
        IDPESSOAFISICA: (typeRegistration === 'CPF') ? 'S' : 'N',
        IDINSTITUICAO: (typeRegistration === 'CNPJ') ? 'S' : 'N',
        CDTIPOENDERECO: Metadata.getInstance<Select>('BPForm-typeAddress').value,
        NRCEPENDERECO: Metadata.getInstance<TextInput>('BPForm-CEP').value,
        DSENDERECO: Metadata.getInstance<TextInput>('BPForm-address').value,
        NRIMOVELENDERECO: Metadata.getInstance<TextInput>('BPForm-number').value,
        DSCOMPLEENDERECO: Metadata.getInstance<TextInput>('BPForm-additionalAddress').value,
        DSREFERENCIAENDE: Metadata.getInstance<TextInput>('BPForm-reference').value,
        CDPAIS: Metadata.getInstance<Select>('BPForm-country').value,
        SGESTADO: Metadata.getInstance<Select>('BPForm-state').value,
        CDMUNICIPIO: Metadata.getInstance<Select>('BPForm-city').value,
        CDBAIRRO: (DISTRICT === null) ? null : DISTRICT.CDBAIRRO,
        CDOPERADOR: localStorage.getItem('CDOPERADOR'),
        NMBAIRROENDERECO: (DISTRICT === null) ? null : DISTRICT.NMBAIRRO,
        CDINSCESTA: Metadata.getInstance<TextInput>('formAdditionalData-INSCESTAD').value,
        CDINSCMUNI: Metadata.getInstance<TextInput>('formAdditionalData-INSCMUNI').value,
      }).then((response: IDictionary) => {
        const returnMessage = response.data.dataset.partner[0];
        if (returnMessage === 'OK') {
          this.alertMessage('Parceiro de Negócio adicionado com sucesso!', 'success');
          this.closeModalNewPartner();
          grid.datasource.get();
        } else {
          this.closeModalNewPartner();
          this.dialogMessage('Operação bloqueada! Este Parceiro de Negócio não pôde ser adicionado.<br><br>Se o problema persistir entre em contato com o suporte.');
        }
      });
    } else {
      this.alertMessage('Por favor, preencha todos os campos obrigatórios(*)!', 'error');
    }
  }

 // ---------------------- Funções da tela de visualização e edição -----------

  // Função que insere o TIPO PRINCIPAL de um parceiro de negócio, conforme selecionado no campo de TIPO DE PARCEIRO.
  public putValueOnSelectOfMainType({ component }: any) {
    const select = (component.name === 'formNewPartner-typeOfPartner') ? Metadata.getInstance<Select>('formNewPartner-mainType') : Metadata.getInstance<Select>('formEditMainType');

    if(!component.value || component.value.length === 0) {
      select.setValue(null);
    } else if(!component.value.includes(select.value)) {
      select.setValue(component.value[0]);
    }
  }

  public mountInformationOfBusinessPartners() {
    const NRPARCNEGOCIO = window.location.search.split('?')[1];

    Http.post('/getSpecificBusinessPartner', { NRPARCNEGOCIO }).then((response: IDictionary) => {
      response.data.dataset.partnerFound.forEach((data: IDictionary) => {
        Metadata.getInstance<Text>('nameOfCorporate').text = data.NMPRINCIPALPARC;
        Metadata.getInstance<Select>('formEditRegistrationType').setValue(data.CDTIPOINSCRICAO);
        Metadata.getInstance<TextInput>('formRegistrationNumber').value = data.NRINSCRICAOPARC;
        Metadata.getInstance<Select>('formEditActive').setValue(data.IDATIVO);
        Metadata.getInstance<Select>('formEditMainType').setValue(data.CDTIPOPARCPRINCIPAL);
        Metadata.getInstance<Select>('formEditPartnerClassification').setValue(data.NRTPCLASSIFICAPARC);
        Metadata.getInstance<Select>('formEditOperator').setValue(data.CDOPERRESPONSAVEL);
        Metadata.getInstance<TextInput>('formEditCorporateName').value = data.NMPRINCIPALPARC;
        Metadata.getInstance<TextInput>('formEditShortName').value = data.NMSECUNDARIPARC;
        Metadata.getInstance<ZdDate>('formEditFoundation').value = data.DTNASCIFUNDPARC;
        Metadata.getInstance<Select>('formEditMarkedSegment').setValue(data.CDSEGMER);
        Metadata.getInstance<Select>('formEditBranchOfActivity').setValue(data.CDATIVCLI);
        Metadata.getInstance<Select>('formEditTypeOfActivity').setValue(data.NRSEQTPATIVCLI);
        Metadata.getInstance<TextInput>('formIncludeIn').value = data.DTINCLUSAO;
        Metadata.getInstance<TextInput>('formIncludeBy').value = data.NMOPERINCLUSAO;
        Metadata.getInstance<TextInput>('formChangedIn').value = data.DTULTATU;
        Metadata.getInstance<TextInput>('formChangedBy').value = data.NMOPERULTATU;
        Metadata.getInstance<TextInput>('formEditAdditionalData-INSCESTAD').value = data.CDINSCESTA;
        Metadata.getInstance<TextInput>('formEditAdditionalData-INSCMUNI').value = data.CDINSCMUNI;
        this.NAMEOFCORPORATE = data.NMPRINCIPALPARC;
        Metadata.getInstance<SelectMultiple>('formEditTypeOfPartner').setValue(data.TIPODEPARCEIRO.split(', '));
      });
    });
  }

  public saveSpecificBusinessPartner() {
    const NRPARCNEGOCIO = window.location.search.split('?')[1];
    const formBP = Metadata.getInstance<Form>('formEditBusinessPartner');
    const typeRegistration = Metadata.getInstance<Select>('formEditRegistrationType').value;

    if (formBP.validate()) {
      Http.post('updateSpecificBusinessPartner', {
        NRPARCNEGOCIO: NRPARCNEGOCIO,
        IDATIVO: Metadata.getInstance<Select>('formEditActive').value,
        CDSEGMER: Metadata.getInstance<Select>('formEditMarkedSegment').value,
        NMPRINCIPALPARC: Metadata.getInstance<TextInput>('formEditCorporateName').value,
        NMSECUNDARIPARC: Metadata.getInstance<TextInput>('formEditShortName').value,
        DTNASCIFUNDPARC: Metadata.getInstance<ZdDate>('formEditFoundation').value,
        NRINSCRICAOPARC : Metadata.getInstance<TextInput>('formRegistrationNumber').value,
        CDOPERADOR : localStorage.getItem('CDOPERADOR'),
        CDTIPOPARCPRINCIPAL: Metadata.getInstance<Select>('formEditMainType').value,
        CDTIPOINSCRICAO: typeRegistration,
        IDPESSOAFISICA: (typeRegistration === 'CPF') ? 'S' : 'N',
        IDINSTITUICAO: (typeRegistration === 'CNPJ') ? 'S' : 'N',
        CDATIVCLI: Metadata.getInstance<Select>('formEditBranchOfActivity').value,
        NRSEQTPATIVCLI: Metadata.getInstance<Select>('formEditTypeOfActivity').value,
        CDOPERRESPONSAVEL: Metadata.getInstance<Select>('formEditOperator').value,
        NRTPCLASSIFICAPARC: Metadata.getInstance<Select>('formEditPartnerClassification').value,
        CDINSCESTA: Metadata.getInstance<TextInput>('formEditAdditionalData-INSCESTAD').value,
        CDINSCMUNI: Metadata.getInstance<TextInput>('formEditAdditionalData-INSCMUNI').value,
        CDTIPOPARCEIRO: Metadata.getInstance<Select>('formEditTypeOfPartner').value,
      }).then((response: IDictionary) => {
        const returnMessage = response.data.dataset.partnerFound[0];
        if (returnMessage === 'OK') {
          this.alertMessage('Parceiro de Negócio alterado com sucesso!', 'success');
          this.mountInformationOfBusinessPartners();
        } else {
          this.dialogMessage('Operação bloqueada! Este Parceiro de Negócio não pôde ser alterado.<br><br>Se o problema persistir entre em contato com o suporte.');
        }
      });
      LoadingService.hide();
    } else {
      this.alertMessage('Por favor, preencha todos os campos obrigatórios(*)!', 'error');
    }
  }

  public validateRegistrationOfTabEdition(){
    const registrationTpe = Metadata.getInstance<Select>('formEditRegistrationType').name;
    const registration = Metadata.getInstance<TextInput>('formRegistrationNumber');
    this.validateRegistration(registrationTpe, registration.name, registration.value);
  }
}
